#!/usr/bin/perl
#blastpairclusterer.plx
use warnings;
use strict;
use List::UtilsBy qw(extract_by);

print "\n";
print "\t\t\tRunning perl version $^V\n" ;
print "\t\t\tProgram Developed by Ryan Christian\n";
print "\t\t\tVersion 1.0 developed 3-28-2017\n";
print "\t\t\tOriginal coding: 3-28-2017\n";
print "\t\t\tLast Modified: 4-12-2017\n";
print "\n\n";
		
print "blastscoreclusterer.plx is a tool designed to check if query-subjects are present in both Forward and Reciprocal BLAST's\n";
print "\tNOTE: this program requires csv files of EACH possible combination of pairs (Self, Forward, Reciprocal BLASTS)\n";
print "\tFor further clustering, use clustermerger5.0.plx\n";
print "\tFor other information, please see program notes\n";		


#program needs
	#"BEST" vs "BETTER"
	#OrthoMCL uses BEST outblasts and BETTER self-blasts to construct networks. 
	#I don't see why we can't also do BETTER outblasts....
	
#program flow
	#for each pairwise BLAST:
		#check to see if the BLAST is reciprocal
			#for BEST option, only use "BEST" outblast files to check.
		#Begin by just marking which ones are positive reciprocals (best and better) in separate files
		#Next, construct array of stringified clusters.
			#for each "BEST" or "BETTER" Blast line, check if either subject or query is already in a cluster; if so, push to that cluster. If not, create new cluster.
#will need to go back through clusters and delete the score ratio when the query and subject is identical (NOT isoforms but the actual ID's are identical)
	

#one thing I could do is write out reciprocal pairwise hits in one combined file, load into an array, reference a master FASTA with ALL of the sequence headers, and then use UtilsBy to condense lines with identical matches. 




print "Please indicate directory of pre-formatted BLAST files: \n";
print "Files should have already been checked to ensure good score ratio.\n";
my $directory = <STDIN>;
chomp $directory;
$directory =~ s/('|")//gi;

if (-d $directory) {
	print "Reading from directory $directory";
}	else {
	die "$directory is not a directory!\n";
}
chdir $directory;

print "Input species to analyze \n";
print "\tNote that species format should match BLAST file format and be separated by spaces.\n";
my $speciesstring = <STDIN>;
chomp $speciesstring;
my @species = split/ /,$speciesstring;
my $speciesnum = scalar @species;

#print "Please input reference file containing all FASTA headers: \n";
#my $reference = <STDIN>;
#chomp $reference;
#$reference =~ s/"//gi;

print "Please indicate output filename:\n";
my $output = <STDIN>; 
chomp $output;
$output =~ s/"//g;




#AaPROT AtPROT BdPROT FvPROT MCPROT OsPROT PpPROT PtPROT PvPROT SbPROT SiPROT SlPROT VvPROT 

#what the program is doing:
	#open SpeciesN by speciesI (ALL hits)
		#save lines to array and close
	#open SpeciesN by SpeciesI (Only best hits; for Inparanoid functionality)
		#save lines to array and close
	#reopen SpeciesI by SpeciesN (All hits)
		#for each line, 
			#if the line is the FIRST instance of a given $query AND the same subject-query pair is found in the reciprocal "BEST" hit file,
				#print to both best and better hit registers
			#if the line is NOT the first instance, 
				#check if the same subject-query pair is found in the reciprocal "ALL" hit file
					#print ONLY to "Betters" if found
#end goal:
	#produce 1 file of all reciprocal hits, regardless of if it is the first/strongest hit or not
	#produce 2nd file of BEST reciprocal hits only
	#produce 3rd file containing EVERY self-hit line (InParanoid uses "better" reciprocal hits for within-species BLASTs)


for (my $i = 0; $i <=($speciesnum-1); $i++) {
	my $n = $i;	#this ensures that the program won't check redudant files (no need to do reciprocal comparison because it's already been checked with the forward direction)
	
	my $bests = "$output\_Best_Reciprocals_$species[$i].csv";
	open (BESTS, ">", $bests) or die "couldn't open file $bests for writing: $!\n";
#	my $betters = "$output\_Better_Reciprocals_$species[$i].csv";
#	open (BETTERS, ">", $betters) or die "couldn't open file $betters for writing: $!\n";
	my $betterself = "$output\_Better_Self_$species[$i].csv";
	open (BETTERSELF, ">", $betterself) or die "couldn't open file $betterself for writing: $!\n";
	
	for ($n; $n <=($speciesnum-1); $n++) {
	
		my $check = "$species[$n]_$species[$i]_ALLHIT.csv";
		open (CHECKS, "<", $check) or die "couldn't open file $check for referencing: $!\n";
		my @checklines = <CHECKS>;

		close CHECKS;
		
		my $checkbest = "$species[$n]_$species[$i]_BESTHIT.csv";
		open (CHECKBEST, "<", $checkbest) or die "couldn't open file $checkbest for referencing: $!\n";
		my @checkbest = <CHECKBEST>;

		close CHECKBEST;
		
		#need to open this second in case it's a self, in which case it would try to reopen an already open filehandle if this were placed above.
		my $blast = "$species[$i]_$species[$n]_ALLHIT.csv";
		open (BLAST, "<", $blast) or die "couldn't open file $blast for reading: $!\n";
		
		my $savedquery = 0;
		my $first = 0;
		while (<BLAST>) {
			my $line = $_;
			chomp $line;
			my @linecontents = split/,/,$line;
			my $query = $linecontents[0];
			my $subject = $linecontents[1];
			my $pid = $linecontents[2];
			if ($query ne $savedquery) {
					$first = 0;
					$savedquery = $query;
			}
			
			if (grep(m/^$subject,$query,/, @checkbest)) {
				
				#need to insert if $query has not yet been seen (first line)
				#if this fails, need to check if $pid is 100.00% (complete match, even if not first match)
				
				if(($first == 0) or ($pid == 100.00)) {
					print BESTS $line . "\n";
#					print BETTERS $line . "\n";
					$first++;
					#if ($i == $n) {
					#	print BETTERSELF $line . "\n";
					#}
				} 	
			}
			if ($i == $n) {
				if (grep(m/^$subject,$query/, @checklines)) {
#				print BETTERS $line . "\n";
					print BETTERSELF $line . "\n";
				}
			} else {
				next;
			}
		}
		@checklines = '';
		shift @checklines;
		@checkbest = '';
		shift @checkbest;
		close BLAST;
	}
	$n = 0;
	print "Finished $species[$i] at " . localtime . "\n";
#	close BETTERS;
	close BESTS;
	close BETTERSELF;
}


#open (BETTERS, "<", $betters) or die "couldn't open file $betters for reading: $!\n";
#my @array = <BETTERS>;
#close BETTERS;
#chomp @array;

#open (REF, "<", $reference) or die "couldn't open file $reference for reading: $!\n";
#For every 1000 sequences, print a status update of where it's at so that I know it's going


#problem: this is taking too long. 
	#I fear that it's basically chewing through the CPU because the arrays are so big. 
	#fix by first creating temporary files using "Extract_by" to create a separate file for each species
		#then, as it's going through "REF", switch files whenever the REF species changes
		#keep the old array in memory


#my $j;
#while (<REF>) {
#	my $fastaheader = $_;
#	chomp $fastaheader;
#	$fastaheader =~ s/>//g;
#	
#	
#
#	#extract_by function to find all array elements containing the header
#	my @extracts = extract_by {m/((,|^|@)$fastaheader,)/} @array;
#	my $joined = join('@',@extracts);
#	push @array, $joined;
#	$j++;
#	if ($j == 1000) {
#		print "Finished merging clusters for $fastaheader at " . localtime . "\n";
#		print $joined . "\n";
#		$j = 0;
#	}
#}
#close REF;
#my $finalout = "$output\_MERGEDresults.csv";
#open (FINALOUT, ">", $finalout) or die "couldn't open final merge file $finalout for writing: $!\n";
#foreach my $element (@array) {
#	print FINALOUT $element;
#	print FINALOUT "\n";
#}
#close FINALOUT;














