#!/usr/bin/perl
#ClusterGO.pl
use Statistics::Basic qw(:all);
use warnings;
use strict;

#this script is designed to examine cluster contents and map GO terms for them.
#separately, this script will calculate a "unity" score for each of the categories (Molecular function, cellular component, biological process) and for all 3 combined for a total of 4 scores.
	#at the very end, will need to use excel's "averageif" feature and calculate the averages and standard deviations for the different cluster populations. 
	
	
print "Please input cluster CSV file to read: \n";
my $cluster = <STDIN>;
chomp $cluster;
$cluster =~ s/"|'//gi;

print "Please input directory of GO-referenced headers to read: \n";
my $directory = <STDIN>;
chomp $directory;
$directory =~ s/"|'//gi;

if (-d $directory) {
	print "Reading from directory $directory.\n";
}	else {
	print "$directory is not a directory! \n";
	die;
}
chdir $directory;

opendir(DIR, $directory) or die "couldn't open directory $directory for reading: $!\n";
my @files = readdir(DIR);
closedir(DIR);
my @csvs = grep(m/.csv$/,@files);

print "Please specify filename for output file:\n";
my $out = <STDIN>;
chomp $out;
$out =~ s/"|'//gi;
$out =~ s/.csv$//i;
$out = "$out.csv";

open (OUT, ">", $out) or die "couldn't open file $out for writing: $!\n";

my @species = qw(Ambtr Antam Artha Bradi Frave Glyma Maldo Orysa Panvi Poptr Prupe Setit Solly Sorbi Vitvi);
my %hash;


foreach my $specie (@species) {
	my @matchfile = grep(m/$specie/,@csvs);
	my $csv = shift @matchfile;
	open (CSV, "<", $csv) or die "couldn't open file $csv for reading; $!";
	while (<CSV>) {
		my $line = $_;
		chomp $line;
		my @split = split/,/,$line;		#files should be organized in "seq,GO terms(semicolon-delimited)" format
		
		unless ($split[0]) {
			next;
		}
		my $go = $split[1];
		unless ($go) {
			$go = "-";
		}
		$go =~ s/\s//ig;
		if ($go !~ m/\w/) {
			$go = "-";
		}
		$hash{$split[0]} = $go;
	}
}

	
open (CLUSTER, "<", $cluster) or die "couldn't open file $cluster for reading: $!\n";
my $i = 0;
while (<CLUSTER>) {
	if ($i == 0) {		#skip header line;
		$i++;
		next;
	}
	my $line = $_;
	chomp $line;
	my @linecontents = split/,/,$line;
	my $cluster = shift @linecontents;
	print OUT $cluster . ",";
	foreach my $header (@linecontents) {
		my $fetch = $header;
		$fetch =~ s/c$//i;
		my $go = $hash{$fetch};	
		unless ($go){
			print "Error with header $header; no GO term found.\n";
		}
		print OUT $header . "\t" . "$go" . ",";
	}
	print OUT "\n";
}
	
close OUT;
close CLUSTER;

open (OUT, "<", $out) or die "couldn't open file $out for reading: $!\n";

my $newout = $out;
$newout =~ s/.csv$/_GOtotal.csv/i;

open (NEWOUT, ">", $newout) or die "couldn't open file $newout for writing: $!\n";
my $stats = $out;
$stats =~ s/.csv$/_GOstats.csv/i;
open (STATS, ">", $stats) or die "couldn't open file $stats for writing: $!\n";
print NEWOUT "Species Population,Clustername,Total Avg,Cellular Component Avg,Biological Function Avg,Molecular Process Avg,Cluster Population\n";
print STATS "Cluster Species Count,Number of Clusters,Total Avg,Total Stdev,Component Avg,Component Stdev,Function Avg,Function Stdev,Process Avg,Process Stdev\n";

while (<OUT>) {
	my $line = $_;
	chomp $line;
	my @split = split/,/,$line;
	my $clustername = shift @split;
	my $clusterpopulation = scalar @split;
	my $speciescount = 0;
	foreach my $spec (@species) {
		if (grep(m/$spec/,@split)) {
			$speciescount++;
		}
	}
	print NEWOUT "$speciescount,$clustername";
	my @goterms;
	foreach my $split (@split) {
		my @sequencesplit = split/\t/,$split;
		my $localgo = $sequencesplit[1];
		my @localgoterms;
		if ($localgo) {
			@localgoterms = split/;/,$localgo;
			push @goterms, @localgoterms;
		} else {
			next;
		}
	}
	@goterms = UNIQUE(@goterms);
	my @finalgoterms = grep(m/:GO:/i, @goterms);	#skip the blank entries	
	my @total;
	my @component;
	my @function;
	my @process;
	
	foreach my $goterm (@finalgoterms) {
		(my $category) = ($goterm =~ m/^(\w)/i);
		my @matches = grep(m/$goterm($|;)/,@split);
		my $scalar = scalar @matches;
		my $percent = $scalar/$clusterpopulation;
		push @total, $percent;
		if ($category eq "C") {
			push @component, $percent;
		} elsif ($category eq "F") {
			push @function, $percent;
		} elsif ($category eq "P") {
			push @process, $percent;
		} else {
			print "Error: GO term $goterm lacking identifiable ontology\n";
			die;
		}
	}
	my $totalavg;
	my $componentavg;
	my $functionavg;
	my $processavg;
	
	if (scalar @total > 0) {
		$totalavg = avg(@total);
	} else {
		$totalavg = "-";
	}
	if (scalar @component > 0) {
		$componentavg = avg(@component);
	} else {
		$componentavg = "-";
	}
	if (scalar @function > 0) {
		$functionavg = avg(@function);
	} else {
		$functionavg = "-";
	}
	if (scalar @process > 0) {
		$processavg = avg(@process);
	} else {
		$processavg = "-";
	}
	my $join = ",$totalavg,$componentavg,$functionavg,$processavg,$clusterpopulation\n";
	print NEWOUT $join;
}

close NEWOUT;
open (NEWOUT, "<", $newout) or die "couldn't open file $newout for reading; $!\n";
my @statslines = (<NEWOUT>);
shift @statslines;

foreach (my $n = 1; $n<=15; $n++) {
	my @t;
	my @c;
	my @f;
	my @p;
	my @locallines = grep(m/^$n,/,@statslines);
	my $scalarclusters = scalar @locallines;
	foreach my $line (@locallines) {
		my @localsplit = split/,/,$line;
		
		unless ($localsplit[2] =~ m/-/) { push @t, $localsplit[2];}
		unless ($localsplit[3] =~ m/-/) { push @c, $localsplit[3];}
		unless ($localsplit[4] =~ m/-/) { push @f, $localsplit[4];}
		unless ($localsplit[5] =~ m/-/) { push @p, $localsplit[5];}
	}
	my $tavg = avg(@t);
	my $tstdev = stddev(@t);
	my $cavg = avg(@c);
	my $cstdev = stddev(@c);
	my $favg = avg(@f);
	my $fstdev = stddev(@f);
	my $pavg = avg(@p);
	my $pstdev = stddev(@p);
	
	print STATS "$n,$scalarclusters,$tavg,$tstdev,$cavg,$cstdev,$favg,$fstdev,$pavg,$pstdev\n";
}



sub UNIQUE {
	my %seen;
    grep !$seen{$_}++, @_;
	my @unique = keys (%seen);
	my $scalar = scalar @_;
	return @unique;
}


