#!/bin/usr/perl
#GeneIsoformChecker.pl
use List::UtilsBy qw(extract_by);
use warnings;
use strict;


print "Please input directory of parent gene ID files to read: \n";
my $directory = <STDIN>;
chomp $directory;
$directory =~ s/"|'//gi;

if (-d $directory) {
	print "Reading from directory $directory.\n";
}	else {
	print "$directory is not a directory! \n";
	die;
}
chdir $directory;

opendir(DIR, $directory) or die "couldn't open directory $directory for reading: $!\n";
my @files = readdir(DIR);
closedir(DIR);
my @parents = grep(m/ParentGeneIDs/,@files);
chdir $directory;

print "Please input query file to read: \n";
my $query = <STDIN>;
chomp $query;
$query =~ s/"|'//gi;

print "Please specify output filename:\n";
my $out = <STDIN>;
chomp $out;
$out =~ s/"|'//gi;


my %conversions;

foreach my $parent (@parents) {
	open (PARENT, "<", $parent) or die "couldn't open file $parent for reading:$!\n";
	while (<PARENT>) {
		my $line = $_;
		chomp $line;
		my @split = split/,/,$line;
		if ($split[0]) {
			$conversions{$split[0]} = $split[1];
		}
	}
	close PARENT;
}

open (OUT, ">", $out) or die "couldn't open file $out for writing:$!\n";
open (QUERY, "<", $query) or die "couldn't open file $query for reading:$!\n";
print OUT "Cluster,cTP Sequence,cTP Parent Gene,non-cTP Sequence,non-cTP Parent Gene,Isoform Status\n";
while (<QUERY>) {
	my $line = $_;
	chomp $line;
	$line =~ s/>//gi;
	my @split = split/,/,$line;
	print OUT "$split[0],";
	my $ctp = $split[1];
	my $ctpclone = $ctp;
	$ctp =~ s/c$//;
	my $non = $split[2];

	my $ctpparent = $conversions{$ctp};
	unless ($ctpparent) {
		$ctpparent = "N/A";
	}
	print OUT "$ctpclone,$ctpparent,";
	my $nonparent = $conversions{$non};
	unless ($nonparent) {
		$nonparent = "N/A";
	}
	print OUT "$non,$nonparent,";
	if ($ctpparent eq $nonparent) {
		print OUT "1\n";
	} else {
		print OUT "0\n";
	}
}
close QUERY;
close OUT;

