#!/usr/bin/perl
#RAxMLExtractor-simplified.pl
use Statistics::Basic qw(:all);
use List::UtilsBy qw(extract_by);
use warnings;
use strict;


#	Split based on commas
#	For each line, count parentheses, and split based on colons
#•	Each parentheses is adding one more branch; will store in an array to determine how much length to add
#•	Each time a back parentheses is reached, need to go back and add branch to the total branch lengths of each of the previous headers until the last forward parentheses. 
#	Shift out first value: this is the header
#	Find if all “ctp” seqs group together: if so, it’s monophyletic. 
#•	NEXT:
#Find all continuous numbers (might be easiest to split based on parentheses and colons)
#•	Add branch lengths
#•	Extract the smallest branch length
#o	This is the “oldest” one: the “branch” point.
#•	Can do this in a nested series of hashes. 
#	1.	Each time a new parentheses is added, add another array. 
	#	Each time a back parenthesis is encountered, add the following branch length to the previous array

	
	
	#detect gain/loss by checking if the smallest scope containing both cTP's and non-cTP's has all or less than all of the total number of cTP's
		#if it is pure gain, the smallest possible scope will contain ALL of the cTP's/
		#if it is gain/loss, the smallest scope will contain less than all of the cTP's
			#in this case, I could optionally write to a third file that's GAIN/LOSS

#simplified version uses concatenated elements of single arrays and splits them up manually each time they're accessed rather than make a needlessly complicated 2-dimensional array with array references

	
	
#basal sequence is the last one in the list
	#if that sequence is a "cTP", I should discard it as this is a case of LOSS of the transit peptide, rather than gain. 	
	
	
	
#check for first/last cTP 
	#reject basal clusters as "gain/loss"
#check for single group of cTP's
	#reject fragemented clusters as "gain/loss"
#find smallest scope containing both cTP and non-ctp
#find smallest cTP and smallest non-cTP
	

print "please input directory of RaxML files in Newick format to read: \n";
my $directory = <STDIN>;
chomp $directory;
$directory =~ s/"|'//gi;

if (-d $directory) {
	print "Reading from directory $directory.\n";
} else {
	print "$directory is not a directory! \n";
	die;
}

print "Please input prefix of the database you're analyzing (eg BrachyPan,Arabidopsis,etc)\n";
my $prefix = <STDIN>;
chomp $prefix;
$prefix =~ s/"|'//gi;

my $out = "NearestNeighborRaxML-$prefix-TransitPeptideGainDetections.csv";
open (OUT, ">", $out) or die "couldn't open file $out for writing: $!\n";
my $poly = "NearestNeighborRaxML-$prefix-polyphyly-detections.csv";
open (POLY, ">", $poly) or die "couldn't open file $poly for writing: $!\n";
my $basal = "NearestNeighborRaxML-$prefix-basal-detections.csv";
open (BASAL, ">", $basal) or die "couldn't open file $basal for writing:$!\n";
my $gainloss = "NearestNeighborRAxML-$prefix-TransitPeptide-GainLossDetections.csv";
open (GAINLOSS, ">", $gainloss) or die "couldn't open file $gainloss for writing:$!\n";

chdir $directory;
opendir(DH, $directory);
my @files = readdir(DH);
closedir(DH);
my @raxes = grep(m/.raxml$/, @files);

#need to implement check for terminal positions
	#terminals shouldn't need to be examined. 

foreach my $rax (@raxes) {
	my $clustername = $rax;
	$clustername =~ s/.afa$//i;
	
	open (RAX, "<", $rax) or die "couldn't open file $rax for reading: $!\n";
	my @raxlines = (<RAX>);
	close RAX;


	my $i = 0;	#this is the positional array
	my $x = 0;	#this is the array iterator

	my $readline = shift @raxlines;
	my @commasplit = split/,/,$readline;
	my $scalarheaders = scalar @commasplit;
	my $last = $commasplit[$scalarheaders-1];
	my @lastsplit = split/:/,$last;
	my $gainlossrecord = 0;
	my $first = $commasplit[0];
	my @firstsplit = split/:/,$first;
	if ($firstsplit[0] =~ m/c$/i) {
		print BASAL "$rax,$firstsplit[0],$lastsplit[0]\n";
	#	print "cluster $clustername has a basal sequence $firstsplit[0] at the beginning\n";
		$gainlossrecord = 1;
	}
	if ($lastsplit[0] =~ m/c$/i) {
		print BASAL "$rax,$firstsplit[0],$lastsplit[0]\n";
	#	print "cluster $clustername has a basal sequence $lastsplit[0] at the end\n";
		$gainlossrecord = 1;
	}
	if ($gainlossrecord) {
		next;
	}
	
	my @activearray;	#this contains the active array numbers
	my %branches;	#this hash stores the branch lengths for each header;
	my @headers; 	#this array stores the order of the phylogenetic tree. 
	my @cTPs;
	my @scope;
	my @totalscope;
	
	print "\n\n$rax\n\n";

	foreach my $comma (@commasplit) {
		$i++;
		my $starts = () = $comma =~ /\(/g;
		my $stops = () = $comma =~ /\)/g;
		
		$comma =~ s/(\(|\))//gi;		
		$comma =~ s/;//;
		chomp $comma;
		my @colsplit = split/:/,$comma;
		my $header = shift @colsplit;
		if ($header =~ m/\w/ig) {
		push @headers, $header;
			if ($header =~ m/c$/i) {				#add to cTP list
				push @cTPs, $i;
			}	
			my $headerbranch = shift @colsplit;			
			$branches{$header} = $headerbranch;			#define the length of the current header's terminal branch
		}
		
		if ($starts) {
			foreach (my $r = 0; $r < $starts; $r++) {
				$x++;
				push @activearray, $x;
			}
			foreach my $active (@activearray) {
				$scope[$active] .= ",$header";		#add to currentscope
				$totalscope[$active] .= ",$header";	#add to total scope
			}
		}
		
		if ($stops) {
			foreach my $active (@activearray) {
				$scope[$active] .= ",$header";		#add to currentscope
				$totalscope[$active] .= ",$header";	#add to total scope
			}		
		
			foreach (my $r = 0; $r < $stops; $r++) {
				
				my $add = shift @colsplit;
				my $active = pop @activearray;				
				my @arraysplit = split/,/,$scope[$active];
				shift @arraysplit;	#remove dead entry before first comma
				foreach my $split (@arraysplit)  {
					$branches{$split} += $add;
				}		
			}
		}
	}
	shift @scope;
	shift @totalscope;
	my $startseq = $headers[0];
	my $endnumber = (scalar @commasplit - 1);
	my $endseq = $headers[$endnumber];
#	if (($startseq =~ m/c$/i) or ($endseq =~ m/c$/i)) {
#		print BASAL "$clustername,$startseq,$endseq\n";
#		next;
#	}
	
	#check for monophyly (clustered cTP's)
	my $matchscalar = scalar @cTPs;
	my $polyphyly = 0;
	if ($matchscalar > 1) {
		for (my $n = 0; $n < ($matchscalar-1); $n++) { 
			if ((($cTPs[$n+1]) - ($cTPs[$n])) == 1) {
				next; 	#indicates position is sequential;
			} else {
				#print "\ncluster $clustername is polyphyletic";
				print POLY "$clustername,";
				print POLY "$cTPs[$n+1],$cTPs[$n]";
				print POLY "\n";
				$polyphyly = 1;
				last;
			}
		}
	}
	if ($polyphyly) {
		next;
	}

	#find all scopes containing cTP's and non-cTP's/
	
#	my $testcount = 0;
#	foreach my $total (@totalscope) {
#	
#		print "\n" . $testcount+1 . "\n" . $total . "\n";
#		$testcount++;
#	}
#	foreach my $head (@headers) {
#		print "$head\t$branches{$head}\n";
#	}
	
	my @cTPseqs = grep(m/(c$|c,)/i,@totalscope);
	unless (@cTPseqs) {
		print "No cTP headers found in $rax: skipping\n";
		next;
	}
	
	my @mixed = grep(m/(\d$|\d,|a$|a,|n$|n,)/, @cTPseqs);
	my %final;
	my $finalcount = 0;
	
	foreach my $mixed (@mixed){
		my $seqs = () = $mixed =~ /,/g;		#because I add a comma before EVERY header in each scope, there are as many seqs as there are commas, and it is far easier to check the commas. 
		$final{$finalcount} = $seqs;
#		print "$finalcount,$seqs\n";
		$finalcount++;
	}
	my @mixedsorted;
	foreach my $key (sort{$final{$a} <=> $final{$b} } keys %final) {
		push @mixedsorted, $key;
	}
	
	#print "\n";
#	foreach my $key (@mixedsorted) {
#		print "$key,$final{$key}\n";  
#	}
	

	my @splitfinal;
	foreach my $remainder (@mixedsorted) {
		my $finalarray = $mixed[$remainder];
		my @splittemp = split/,/,$finalarray;
		shift @splittemp;
		my @cTPseqs = grep(m/c$/,@splittemp);
		if (scalar @cTPseqs < $matchscalar) {
			#print scalar @cTPseqs . ",";
			#print $matchscalar . "\n";
			$gainlossrecord = 1;
			next;
		} elsif (scalar @cTPseqs == $matchscalar) {
			@splitfinal = @splittemp;
			last;
		}
	}
	
	#implement loop to find shortest cTP, shortest non-cTP.
	my @ctpfinal = extract_by{m/c$/} @splitfinal;
	my @nonfinal = @splitfinal;
	
	unless ((scalar @ctpfinal > 0) && (scalar @nonfinal > 0)) {
		next;
	}

	my $shortestctp;
	my $shortestnon;
	
	my $w = 0;
	foreach my $finalctp (@ctpfinal) {
		#print "CTP loop \n";
		if ($w == 0) {
			$shortestctp = $finalctp;
			#print "START: $finalctp,$shortestctp,$branches{$finalctp}\n";
			$w++;
		}
		if ($branches{$shortestctp} > $branches{$finalctp}) {
			$shortestctp = $finalctp;
			#print "NEXT: $finalctp,$shortestctp,$branches{$finalctp}\n";
		}
	}
	
	$w = 0;
	foreach my $finalnon (@nonfinal) {
		#print "non loop \n";
		if ($w == 0) {
			$shortestnon = $finalnon;
			#print "$finalnon,$shortestnon\n";
			$w++;
		}	
		if ($branches{$shortestnon} > $branches{$finalnon}) {
			$shortestnon = $finalnon;
			#print "$finalnon,$shortestnon\n";
		}	
	}

	if ($gainlossrecord) {
		print GAINLOSS "$clustername,";
		foreach my $finalctp (@ctpfinal) {
			print GAINLOSS "$finalctp,";
		}
		foreach my $finalnon (@nonfinal) {
			print GAINLOSS "$finalnon,";
		}
	}

	#print OUT $clustername, $header 1, branch length 1, $header 2, branch length 2

	print OUT "$clustername,$shortestctp,$branches{$shortestctp},$shortestnon,$branches{$shortestnon}\n";
}
close OUT;
close GAINLOSS;
close POLY;


#easier method: capture every cTP in a row and then push/pop