#!/usr/bin/perl
#blastscoreclusterer.plx
use warnings;
use strict;
use List::UtilsBy qw(extract_by);

print "\n";
print "\t\t\tRunning perl version $^V\n" ;
print "\t\t\tProgram Developed by Ryan Christian\n";
print "\t\t\tVersion 1.0 developed 3-28-2017\n";
print "\t\t\tOriginal coding: 3-28-2017\n";
print "\t\t\tLast Modified: 4-12-2017\n";
print "\n\n";
		
print "blastscorechecker.plx is a tool designed to cluster results of whole proteome pairwise BLAST alignments\n";
print "\tNOTE: this program requires csv files of EACH possible combination of pairs (Self, Forward, Reciprocal BLASTS)\n";
print "\tFor other information, please see program notes\n";		

# Part 1: program flow
	#input possible species
	#generate pair data
		#store selfs in separate array
	#run through selfs
		#condense fragments; add scores
		#throw out in score ratio <40% (user defined)
		#determine BEST hit lines
			#save out BEST hit file
			#save out ALL hit file
	#run through forwards 
		#load BEST hit lines into array in memory
		#use iterator value to progress through array
		#use WHILE loop to progress through forwards
			#condense forward lines
			#throw out if score ratio <40% (user defined)
			#mark BEST hit lines
				#save out BEST hit lines
				#save out ALL hit lines
				
print "Please indicate directory of pairwise BLAST files: \n";
my $directory = <STDIN>;
chomp $directory;
$directory =~ s/"//gi;

if (-d $directory) {
	print "Reading from directory $directory";
}	else {
	die "$directory is not a directory!\n";
}
chdir $directory;

REDO:
print "Define a cutoff threshold for score ratio- number should be between 0 and 1: \n";
my $scorecutoff = <STDIN>;
chomp $scorecutoff;
if ($scorecutoff > 1){
	print "Incorrect format! \n";
	goto REDO;
}


print "Please input species to perform reciprocal BLAST tests on: \n";
print "\n\tInput is case sensitive. Separate inputs by spaces.\n";
print "\tSpecies should be in the format 'XxCTP' or 'XxPROT'\n";

my $speciesinput = <STDIN>;
chomp $speciesinput;
my @species = split/ /, $speciesinput;
my $speciesnum = scalar @species;

#opendir (DIR, $directory) or die "couldn't open directory $directory for reading: $!\n";
#my @files = readdir(DIR);
#closedir(DIR);
#my @csvs = grep(m/.csv$/, @files);

for (my $i = 0; $i <= ($speciesnum-1); $i++) {
	my $selfblast = "$species[$i]\_$species[$i].csv";
	open (SELFBLAST, "<", $selfblast) or die "couldn't open file $selfblast for reading: $!\n";
	my $bestself = "$species[$i]\_$species[$i]\_BESTHIT.csv";
	my $allself = "$species[$i]\_$species[$i]\_ALLHIT.csv";
	open (BESTSELF, ">", $bestself) or die "couldn't open file $bestself for writing: $!\n";
	open (ALLSELF, ">", $allself) or die "couldn't open file $allself for writing: $!\n";
	my $savedquery = 0;
	my $savedsubject = 0;
	my $savedscore = 0;
	my $bestscore = 0;
	my $num = 0;
	my $scoreratio = 0;
	
	while (<SELFBLAST>) {
		my $line = $_;
		chomp $line;
		my @lineexplode = split/,/, $line;
		#check if query is different:
		if ($savedquery ne $lineexplode[0]) {
			#if new query IS different from the last query, print off the saved information and set the values to the current line
			if ($savedquery ne "0") {	#skip printing undefined value on first line; should return "undefined" if not defined. 
				if ($num == 0) {	#check if there was only one entry for the last query
					$bestscore = $savedscore;
					print BESTSELF "$savedquery,$savedsubject,$savedscore,$bestscore\n";
				}
				$scoreratio = ($savedscore/$bestscore);

				if ($scoreratio >= $scorecutoff) {
					print ALLSELF "$savedquery,$savedsubject,$savedscore,$bestscore,$scoreratio\n";		
				}
				$savedquery = $lineexplode[0];
				$savedsubject = $lineexplode[1];
				$savedscore = $lineexplode[2];
				$num = 0;
			} else { #this is first line only. 
				$savedquery = $lineexplode[0];
				$savedsubject = $lineexplode[1];
				$savedscore = $lineexplode[2];
			}
		} else {	#checking now to see if subject is the same
		
			#check if query-subject is the same:
			if ($savedsubject eq $lineexplode[1]) {
				$savedscore = ($savedscore + $lineexplode[2]);
			} else {
				if ($num == 0) {
					$bestscore = $savedscore;
					chomp $bestscore;
					chomp $savedscore;
					print BESTSELF "$savedquery,$savedsubject,$savedscore,$bestscore\n";
				}
				#check score ratio; is it higher than user defined threshold?
				$scoreratio = ($savedscore/$bestscore);
				if ($scoreratio >= $scorecutoff) {
					chomp $scoreratio;
					chomp $bestscore;
					chomp $savedscore;
					print ALLSELF "$savedquery,$savedsubject,$savedscore,$bestscore,$scoreratio\n";
				}
				#check if this is the first subject-query match (no prior matches for this pair)
				#if so, save as the BEST score
				
				$savedquery = $lineexplode[0];
				$savedsubject = $lineexplode[1];
				$savedscore = $lineexplode[2];
				$num++;
			}
		}
	}
	#finish by printing off last line
	if ($num == 0) {
		$bestscore = $savedscore;
		print BESTSELF "$savedquery,$savedsubject,$savedscore,$bestscore\n";
	}
	$scoreratio = ($savedscore/$bestscore);
	if ($scoreratio >= $scorecutoff) {
		print ALLSELF "$savedquery,$savedsubject,$savedscore,$bestscore,$scoreratio\n";
	}
	close ALLSELF;
	close BESTSELF;
	close SELFBLAST;
	print "Finished Processing $species[$i] vs $species[$i] at " . localtime . "\n";
	

	
#============================================================================================================================================
#											OUTBLASTING BEGIN:
#============================================================================================================================================	
		
	open (BESTSELF, "<", $bestself) or die "couldn't open file $bestself for reading: $!\n";
	my @bestselflines = <BESTSELF>;
	close BESTSELF;
	chomp @bestselflines;
	
	for (my $n = 0; $n <= ($speciesnum-1); $n++) {
		if ($n == $i) {
			next;
			#skip if the query and subject are the same (already done in step 1)
		}
		
		my $outblast = "$species[$i]\_$species[$n].csv";
		open (OUTBLAST, "<", $outblast) or die "couldn't open file $outblast for reading: $!\n";
		my $bestout = "$species[$i]\_$species[$n]\_BESTHIT.csv";
		open (BESTOUT, ">", $bestout) or die "couldn't open file $bestout for writing: $!\n";
		my $allout =  "$species[$i]\_$species[$n]\_ALLHIT.csv";
		open (ALLOUT, ">", $allout) or die "couldn't open file $allout for writing: $!\n";
		
		my $reference = 0;
		$savedquery = 0;
		$savedsubject = 0;
		$savedscore = 0;
		$num = 0;
		my $currentref = 0;
		my @referenceexplode = '';
		
		while (<OUTBLAST>) {
			my $line = $_;
			chomp $line;
			my @lineexplode = split/,/, $line;
			

			
			#first check to see if line query doesn't match previous line's query
			#if it doesn't, print out line to the appropriate OUT files, and reassign variables to correct values
			if ($lineexplode[0] ne $savedquery) {
				if ($savedquery ne "0") {
					#print out last line
					#check score ratio; is it higher than user defined threshold?
					$scoreratio = ($savedscore/$referenceexplode[2]);
					if ($scoreratio >= $scorecutoff) {
						if ($num == 0) {
							print BESTOUT "$savedquery,$savedsubject,$savedscore,$currentref,$scoreratio\n";
						} 
						print ALLOUT "$savedquery,$savedsubject,$savedscore,$currentref,$scoreratio\n";
					}
					$savedquery = $lineexplode[0]; #save current line's contents to save variables
					$savedsubject = $lineexplode[1];
					$savedscore = $lineexplode[2];	
				} else {
					$savedquery = $lineexplode[0];
					$savedsubject = $lineexplode[1];
					$savedscore = $lineexplode[2];
				}
				
				my $countdown = 0; 	#this initializes an autodestruct value that will allow ONE extra search through the array before it terminates; serves to check if the array is out of order compared to the BLAST. Unlikely, but hey- just a check on the system. 


				#first, start looking for the correct reference query to match the saved line's query
				REFERENCE: 
				if ($bestselflines[$reference] !~ m/(^$savedquery,)/gi) {
					$reference++;
					if ($reference > (scalar @bestselflines)) {
						if ($countdown == 1) {
							die "couldn't find $savedquery in $species[$i] vs $species[$n]";
							#autodestruct line- stops the program if the $currentquery couldn't be found in @bestselflines after two iterations (one partial, one full)
						} else {
							$reference = 0;
							$countdown = 1;
						}
					}
					goto REFERENCE;
					#purpose of this loop is to just circle through until it finds the correct reference value
				} else {
					@referenceexplode = split/,/,$bestselflines[$reference];	#save matched reference line to array
					$currentref = $referenceexplode[2];
					$num = 0;	#reset $num to 0 because we're on a new query.
				}
			} elsif ($lineexplode[1] eq $savedsubject) {
				$savedscore = ($savedscore + $lineexplode[2]);
			} else {
				$scoreratio = ($savedscore/$referenceexplode[2]);
				if ($scoreratio >= $scorecutoff) {
					if ($num == 0) {
						print BESTOUT "$savedquery,$savedsubject,$savedscore,$currentref,$scoreratio\n";
					} 
					#ALWAYS print to the allout file as long as the score is high enough.
					print ALLOUT "$savedquery,$savedsubject,$savedscore,$currentref,$scoreratio\n";
				}
				$savedquery = $lineexplode[0];
				$savedsubject = $lineexplode[1];
				$savedscore = $lineexplode[2];
				$num++;
			}
		}	
		#finish by printing off last line
		$scoreratio = ($savedscore/$referenceexplode[2]);
		if ($scoreratio >= $scorecutoff) {
			if ($num == 0) {
				print BESTOUT "$savedquery,$savedsubject,$savedscore\n";
			} 
			print ALLOUT "$savedquery,$savedsubject,$savedscore\n";
		}
		print "Finished Processing $species[$i] vs $species[$n] at " . localtime . "\n";
		close ALLOUT;
		close BESTOUT;
		close OUTBLAST;
	}
}