#!/bin/usr/perl
#mutationchecker.pl
use List::UtilsBy qw(extract_by);
use warnings;
use strict;


#this script will scan MSA's for Interspecies, Arabidopsis1001, BrachyPan to search for SNPs and InDels

#represent amino acids in condensed array format (like [0;1;40;2;5432;120;...)
#Calculate SNP insertions (cTP allele for each SNP)
#Calculate SNP deletions (non-cTP allele for each SNP)
#Calculate total insertions (SNPs + Indels)
#Calculate total deletions (SNPs + Indels)
	#find nonsense mutations if the total length of either sequence is less than 100 positions. 

#position trackers
	#alignment position
	#cTP position
	#non-cTP position


my @aminoacids = qw(A C D E F G H I K L M N P Q R S T V W Y);
	
print "Please input directory of aligned sequence pairs to read: \n";
my $directory = <STDIN>;
chomp $directory;
$directory =~ s/"|'//gi;

if (-d $directory) {
	print "Reading from directory $directory.\n";
}	else {
	print "$directory is not a directory! \n";
	die;
}
chdir $directory;

opendir(DIR, $directory) or die "couldn't open directory $directory for reading: $!\n";
my @files = readdir(DIR);
closedir(DIR);
my @alignments = grep(m/.afa$/,@files);

print "Please input name for output files: \n";
my $out = <STDIN>;
chomp $out;
$out =~ s/"|'//gi;

my $total = "$out-TransitPeptideMutations.csv";
my $first = "$out-FirstThirdMutations.csv";
my $second = "$out-SecondThirdMutations.csv";
my $third = "$out-FinalThirdMutations.csv";
my $mature = "$out-MatureMutations.csv";
open (TOTAL, ">", $total) or die "couldn't open file $total for writing:$!\n";
print TOTAL "Cluster,Plastidial Seq, Nonplastidial Seq, Alignment Length, Duplication,Alternative Start,Sequential Indels,ctp identity,mature identity,Insertions,Deletions,SNPs\n";
open (FIRST, ">", $first) or die "couldn't open file $first for writing:$!\n";
print FIRST "Cluster,Plastidial Seq,Nonplastidial Seq,Insertions,Deletions,SNPs\n";
open (SECOND, ">", $second) or die "couldn't open file $second for writing:$!\n";
print SECOND "Cluster,Plastidial Seq,Nonplastidial Seq,Insertions,Deletions,SNPs\n";
open (THIRD, ">", $third) or die "couldn't open file $third for writing:$!\n";
print THIRD "Cluster,Plastidial Seq,Nonplastidial Seq,Insertions,Deletions,SNPs\n";
open (MATURE, ">", $mature) or die "couldn't open file $mature for writing:$!\n";
print MATURE "Cluster,Plastidial Seq,Nonplastidial Seq,Insertions,Deletions,SNPs\n";
my $final = "$out-FinalStatistics.csv";
open (FINAL, ">", $final) or die "couldn't open file $final for writing:$!\n";
print FINAL "Cluster,Plastidial Seq, Nonplastidial Seq,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6\n";
print FINAL "Cluster,Plastidial Seq, Nonplastidial Seq,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,V,W,Y,,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,V,W,Y,,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,V,W,Y,,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,V,W,Y,,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,V,W,Y,,A,C,D,E,F,G,H,I,K,L,M,N,P,Q,R,S,T,V,W,Y\n";
	#1: SNP inserts
	#2: SNP deletes
	#3: indel inserts
	#4: indel deletes
	#5: total inserts
	#6: total deletes



foreach my $alignment (@alignments) {
	
	my $geneduplication = 0;
	 
	#will probably also need to use downstream sequence similarity as a measure for this. 
	my @indels;
	my @tpindels;
	my @matureindels;
	my @totalindels;
	my @snps;
	my @maturesnps;
		#denote insertions with "+"
		#denote deletions with "-"

		#if the gapped sequence alternates between the two, it's likely an alternative splice. 
		
	open (ALIGN, "<", $alignment) or die "couldn't open file $alignment for reading: $!\n";
	print "Working on alignment $alignment\n";
	my @lines = (<ALIGN>);
	my %hash;
	my @headers;
	my $last = '';
	foreach my $line (@lines) {
		chomp $line;
		if ($line =~ m/^>/) {
			$hash{$line} = '';
			$last = $line;
			push @headers, $line;
		} else {
			$hash{$last} .= $line;

		}
	}

	
	close ALIGN;
	chomp @lines;

	my @ctps = extract_by{m/c$/} @headers;
	my $ctp = shift @ctps;

	
	my $ctpspecies = $ctp;
	$ctpspecies =~ s/_seq\d+c//i;
	my $nonctp = shift @headers;
	my $nonspecies = $nonctp;
	$nonspecies =~ s/_seq\d+//i;
	my $ctpseq = $hash{$ctp};
	my $nonctpseq = $hash{$nonctp};
	if ($ctpspecies eq $nonspecies) {
		$geneduplication++;
	}
	
	my $filename = $alignment;
	$filename =~ s/.afa//gi;
	print TOTAL "$filename,$ctp,$nonctp,";
	print FIRST "$filename,$ctp,$nonctp,";
	print SECOND "$filename,$ctp,$nonctp,";
	print THIRD "$filename,$ctp,$nonctp,";
	print MATURE "$filename,$ctp,$nonctp,";
	print FINAL "$filename,$ctp,$nonctp,";
	

	my @ctpsplit = split('',$ctpseq);
	my @nonsplit = split('',$nonctpseq);

	
	my @gap;
	
	if ((scalar @ctpsplit) != (scalar @nonsplit)) {
		print "Error: unequal number of positions in $ctp and $nonctp for file $alignment\n";
		die;
	}
	
	my $i = 0;	#position iterator
	my $r = 0;	#indel iterator
	my $matureident = 0;
	my $ctpident = 0;
	my $maturepercent = 0;
	my $ctppercent = 0;
	my $lastgap = 0;
	my $lastindel = '';
	my $test = $ctpseq;
	$test =~ s/-//gi;
	
	my $ctpend = length $test;
	
	#1: SNP inserts
	#2: SNP deletes
	#3: indel inserts
	#4: indel deletes
	#5: total inserts
	#6: total deletes
	my %snpinsert;
	my %snpdelete;
	my %indelinsert;
	my %indeldelete;
	my %totalinsert;
	my %totaldelete;
	
	my %maturesnpinsert;
	my %maturesnpdelete;
	my %matureindelinsert;
	my %matureindeldelete;
	my %matureinserttotal;
	my %maturedeletetotal;
	
	foreach my $acid (@aminoacids) {		#keeps track of amino acid insertions/deletions/SNPs
		$snpinsert{$acid} = 0; 
		$snpdelete{$acid} = 0; 
		$indelinsert{$acid} = 0; 
		$indeldelete{$acid} = 0; 
		$totalinsert{$acid} = 0; 
		$totaldelete{$acid} = 0; 
		$maturesnpinsert{$acid} = 0;
		$maturesnpdelete{$acid} = 0;
		$matureindelinsert{$acid} = 0;
		$matureindeldelete{$acid} = 0;
		$matureinserttotal{$acid} = 0;
		$maturedeletetotal{$acid} = 0;
	}
	
	my $altstart = 0;			#determines if a start position is aligned on an internal methionine on the sister sequence
	my $ctppositioncount = 0;
	my $nonpositioncount = 0;
	my $alignpositioncount = 0;
	my $maturectplength = 0;
	my $ctplength = 0;
	my $maturenonlength = 0;
	my $matureproteinscore = 0;
	my $maturetrigger = 0;					#this will be the "trigger" for if the alignment has passed the transit peptide.
	my $ctpalign = 0;	#counts the number of aligned residues (skips indels)
	my $totalalign = 0;
	
	foreach my $position (@ctpsplit) {
		my $ctpposition = $position;
		my $nonposition = $nonsplit[$i];
		unless (($ctpposition) && ($nonposition)) {	#should break out if it encounters a blank spot
			print "Unaligned sequence detected at position $ctppositioncount and $nonpositioncount of file $alignment \n";
			last;
		}		
		$alignpositioncount++;
		$i++;
		if ($ctpposition =~ m/\w/) {
			$ctppositioncount++;
		}
		if ($nonposition =~ m/\w/) {
			$nonpositioncount++;
		}
		
		if ((($nonpositioncount) == 1) and ($ctppositioncount > 1)) {
			if (($ctpposition eq $nonposition) and ($ctpposition eq "M")) {
				$altstart = 1;
			}
		}
		if ((($ctppositioncount) == 1) and ($nonpositioncount > 1)) {
			if (($ctpposition eq $nonposition) and ($ctpposition eq "M")) {
				$altstart = 1;
			}
		}

		if (($ctppositioncount > 60)) {	#this triggers the program to end stats for the cTP and start collecting stats for the mature protein
			$maturetrigger = 1;
		} else {
			$ctplength++;
		}
		if (($ctpposition eq $nonposition) | ($ctpposition =~ m/x/i) | ($nonposition =~ m/x/i)) {
			if (($ctpposition =~ m/-/) && ($nonposition =~ m/-/)){
				next;
			}
			if ($maturetrigger) {
				$matureident++;
				$totalalign++;
			} else {
				$ctpident++;
				$ctpalign++;
			}
		
			if ($lastgap == 1) {				#this is the iterator for the indels array.	
				my $lastspot = $ctppositioncount-1;
				$indels[$r] .= ",$lastspot";
				my @indeltest = split/,/,$indels[$r];
				if (($indeltest[1] <= 60) | ($indeltest[3] <=60)) {
					push @tpindels, $indels[$r];
				} else {
					push @matureindels, $indels[$r];
				}
				$r++;
				$lastgap = 0;
			}
			next;
			$lastindel = '';
		} elsif ($ctpposition =~ m/-/) {					#indel detection loop: NOTE: I am only concerned about the cTP position here; sice I'll be extracting which sections of the TP have what indels/snps, it doesn't make sense to look at the non-ctp position.
			if (scalar @indels == 0) {	#create first indel
				#print "First indel triggered\n";
				$indels[$r] = "-,$ctppositioncount,$nonposition";
				unless ($maturetrigger) {
					if ($nonpositioncount <= 60) {
						$indeldelete{$nonposition}++;
						$totaldelete{$nonposition}++;
					} else {
						$maturedeletetotal{$nonposition}++;
						$matureindeldelete{$nonposition}++;				
					}
				} else {
					$maturedeletetotal{$nonposition}++;
					$matureindeldelete{$nonposition}++;
				}
				$lastindel = "-";
				$lastgap = 1;
			} elsif (($lastindel eq "-") && ($lastgap == 1)) {					#continuing a deletion
				#print "continuing deletion\n";
				$indels[$r] .= "$nonposition";
				unless ($maturetrigger) {
					if ($nonpositioncount <= 60) {
						$indeldelete{$nonposition}++;
						$totaldelete{$nonposition}++;
					} else {
						$maturedeletetotal{$nonposition}++;
						$matureindeldelete{$nonposition}++;				
					}
				} else {
					$maturedeletetotal{$nonposition}++;
					$matureindeldelete{$nonposition}++;
				}
				$lastgap = 1;
				$lastindel = "-";
			} elsif (($lastindel =~ m/(\+)/) or ($lastgap == 0)) {				#starting a new deletion
				#print "New Deletion\n";
				
				if ($lastgap == 1) {				#this is the iterator for the indels array.	
					my $lastspot = $ctppositioncount-1;
					$indels[$r] .= ",$lastspot";
					my @indeltest = split/,/,$indels[$r];
					if (($indeltest[1] <= 60) | ($indeltest[3] <=60)) {
						push @tpindels, $indels[$r];
					} else {
						push @matureindels, $indels[$r];
					}
					$r++;
				}
				
				$indels[$r] = "-,$ctppositioncount,$nonposition";
				unless ($maturetrigger) {
					if ($nonpositioncount <= 60) {
						$indeldelete{$nonposition}++;
						$totaldelete{$nonposition}++;
					} else {
						$maturedeletetotal{$nonposition}++;
						$matureindeldelete{$nonposition}++;				
					}
				} else {
					$maturedeletetotal{$nonposition}++;
					$matureindeldelete{$nonposition}++;
				}
				$lastgap = 1;
				$lastindel = "-";
			}
		} elsif ($nonposition =~ m/-/) {
			if (scalar @indels == 0) {			#make first indel
				#print "First indel triggered\n";
				$indels[$r] = "+,$ctppositioncount,$ctpposition";
				unless ($maturetrigger) {
					$indelinsert{$ctpposition}++;
					$totalinsert{$ctpposition}++;
				} else {
					$matureinserttotal{$ctpposition}++;
					$matureindelinsert{$ctpposition}++;
				}
				$lastgap = 1;
				$lastindel = "+";
			} elsif (($lastindel eq "+") && ($lastgap == 1)) {					#continuing an insertion
				#print "continuing insertion\n";
				$indels[$r] .= "$ctpposition";
				unless ($maturetrigger) {
					$indelinsert{$ctpposition}++;
					$totalinsert{$ctpposition}++;
				} else {
					$matureinserttotal{$ctpposition}++;
					$matureindelinsert{$ctpposition}++;
				}
				$lastgap = 1;
				$lastindel = "+";		
			} elsif (($lastindel =~ m/(-)/) or ($lastgap == 0)) {				#starting a new deletion
				#print "New Insertion\n";
				if ($lastgap == 1) {				#this is the iterator for the indels array.	
					my $lastspot = $ctppositioncount-1;
					$indels[$r] .= ",$lastspot";
					my @indeltest = split/,/,$indels[$r];
					if (($indeltest[1] <= 60) | ($indeltest[3] <=60)) {
						push @tpindels, $indels[$r];
					} else {
						push @matureindels, $indels[$r];
					}
					$r++;
				}
				$indels[$r] = "+,$ctppositioncount,$ctpposition";
				unless ($maturetrigger) {
					$indelinsert{$ctpposition}++;
					$totalinsert{$ctpposition}++;
				} else {
					$matureinserttotal{$ctpposition}++;
					$matureindelinsert{$ctpposition}++;
				}
				$lastgap = 1;
				$lastindel = "+";	
			}
		} else {	#both should be a residue at this point
			$ctpalign++;
			$totalalign++;
		
		
			if ($lastgap == 1) {				#this is the iterator for the indels array.	
				my $lastspot = $ctppositioncount-1;
				$indels[$r] .= ",$lastspot";
				my @indeltest = split/,/,$indels[$r];
				if (($indeltest[1] <= 60) | ($indeltest[3] <=60)) {
					push @tpindels, $indels[$r];
				} else {
					push @matureindels, $indels[$r];
				}
				$r++;
				$lastgap = 0;
			}

			$lastindel = '';
			
			if (($ctpposition or $nonposition) eq "X") {
				next;	#no use trying to do substitutions for unknown amino acids
			}
			my $substitution = SNP($ctpposition,$nonposition);	#will have three results: column 1 is whether it is conserved or not, column 2 is the cTP amino acid class, and column 3 is the non-cTP amino acid class.
		
			unless ($maturetrigger) {
				$snpinsert{$ctpposition}++;							#Add to snp additions of the new ctp amino acid
				$totalinsert{$ctpposition}++;						#add to total additions for that amino acid
				$snpdelete{$nonposition}++;							#add to snp deletions of the old non-ctp amino acid
				$totaldelete{$nonposition}++;						#add tot total delections for that aminio acid	
				push @snps, "$ctppositioncount,$ctpposition,$nonposition,$substitution";
			} else {
				$maturesnpinsert{$ctpposition}++;
				$maturesnpdelete{$nonposition}++;
				$matureinserttotal{$ctpposition}++;
				$maturedeletetotal{$ctpposition}++;
				push @maturesnps, "$ctppositioncount,$ctpposition,$nonposition,$substitution";
			}
		}	
	}
	unless (($ctpalign|$ctpident) == 0) {
		$ctppercent = $ctpident/$ctpalign*100;
	} else {
		$ctppercent = '';
	}
#	unless (($totalalign|$matureident) == 0) {
#		$maturepercent = $matureident/($totalalign-$ctpalign)*100;
#	} else {
#		$maturepercent = '';
#	}
	
	
	
	my $sequential = 0;		#this will determine if there is an instance where an insertion and a deletion are back-to-back (within 5 positions to account for partial alignments)
							#according to Parent et al., 1987 and Sarkarhar et al., 2004, the minimum exon size for efficient splicing is 20 positions. 
									#parent et al., 1987: 20 base pairs gives 50% efficiency of splicing
							#https://link.springer.com/article/10.1007/s10709-007-9139-4 (Gudlaugsdottir, 2007)
								#5th percentile MINIMUM is 29 base pairs for exons. Mammals are higher. They say many of these are likely annotation errors and they got rid of them. 
								#I'd say that 30 ought to be the cutoff for me. Good conservative number. 
							#file:///C:/Users/Ryan%20C/Downloads/srep18087.pdf (Guo and Liu, 2015)
								#exons as small as 1 bp are biologically relevant, but probably rare
	
	my $nonsense = 0;
	if (($ctppositioncount < 60) && ($ctppositioncount < $nonpositioncount)) {
			#shift out last indel and label CTP sequence as having a nonsense mutation at the first position
		$nonsense++;
		my $indelmatch = pop @indels;
		chop $indelmatch; 	#remove +/-
		chop $indelmatch;	#remove semicolon
		my @nonsensesplit = split/;/,$indelmatch;
		my $nonsenseposition = shift @nonsensesplit;
	} 
	if (($nonpositioncount < 60) && ($nonpositioncount < $ctppositioncount)) {
			#shift out last indel and label non-CTP sequence as having a nonsense mutation at the first position
		$nonsense++;	
		my $indelmatch = pop @indels;
	}		
	
	my @largeindels;
	my @sequentials;
	foreach my $indel (@tpindels) {
		#calculate length;
			#check for sequential indels after all indels accounted for. 
		my $indellength = 0;
		my @indelsplit = split/;/,$indel;
		my $indeltype = shift @indelsplit;
		my $start;
		my $end;
		
		foreach my $split (@indelsplit) {
			$indellength++;
			my @subsplit = split/,/,$split;
			if ($indellength == 1) {
				$start = $subsplit[0];
			} elsif ($indellength == (scalar @indelsplit)) {
				$end = $subsplit[0];
			}
		}
		if ($indellength >= 10) {
			push @largeindels, $start . ",";
			push @largeindels, $end;
		}
		my $lastend = 0;		
		
		if (scalar @largeindels > 2) {
			shift @largeindels;	#first start is useless
			for (my $s = 1; $s < ((scalar @largeindels)/2); $s++) {
				if (($largeindels[$s+1] - $largeindels[$s]) < 5 ) {
					$sequential++;
					shift @largeindels;
					shift @largeindels;	#shift twice to get a new end/start pair. 
				} else {
					shift @largeindels;
					shift @largeindels;
				}
			}
		}
	}
		
	#Alignment Length, Duplication, Alternative Start, Sequential Indels,
	print TOTAL "$i,$geneduplication,$altstart,$sequential,$ctpident,$matureident,";
	
	#PRINT COMMANDS!!!
	#if (scalar @indels) {foreach my $indel (@indels) {print $indel;	print "\n";	}}
	
	
	my @insertions;
	my @deletions;
	if (@tpindels) {
		foreach my $indel (@tpindels) {
		
			#order: insert/delete (ctp +/non -), start position, end position, length, sequence.
				
			my @indelsplit = split/,/,$indel;
			#	local $, = ",";
			#	print @indelsplit;
		#	print "\n";
			my $orientation = $indelsplit[0];
			my $startposition = $indelsplit[1];
			my $endposition = $indelsplit[3];
			if ($endposition > 60) {
				$endposition = 60;
			}
			my $length = length $indelsplit[2];
			my $join = "$orientation,$startposition,$endposition,$length";
			
			if ($orientation =~ m/-/) {
				push @deletions, $join;
			} else {
				push @insertions, $join;
			}
		}
	}
	my @matureinsertions;
	my @maturedeletions;
	foreach my $maturein (@matureindels) {
		my @indelsplit = split/,/,$maturein;
		my $orientation = $indelsplit[0];
		my $startposition = $indelsplit[1];
		my $endposition = $indelsplit[3];
		my $length = length $indelsplit[2];
		my $join = "$orientation,$startposition,$endposition,$length";
		if ($orientation =~ m/-/) {
			push @maturedeletions, $join;
		} else {
			push @matureinsertions, $join;
		}		
	}
	
	if (@insertions) {
		foreach my $insert (@insertions) {
			my @secondsplit = split/,/,$insert;
			$insert =~ s/,/:/g;
			print TOTAL "$insert;";
			
			if (($secondsplit[1]<= 20) |($secondsplit[2]<=20)) {
				print FIRST "$insert;";
			} elsif ((($secondsplit[1]|$secondsplit[2]) > 20) or (($secondsplit[1]|$secondsplit[2]) <= 40)) {
				print SECOND "$insert;";
			} elsif (($secondsplit[1] >= 40) && ($secondsplit[1] <= 60)) {		#this should only print here if the indel starts in the final third, but it can end afterward. 
				print THIRD "$insert;";
			} 
		}
		foreach my $matureinsert (@matureinsertions) {
			$matureinsert =~ s/,/:/g;
			print MATURE "$matureinsert;";
		}
	}
	print TOTAL ",";
	print FIRST ",";
	print SECOND ",";
	print THIRD ",";
	print MATURE ",";
	if (@deletions) {
		foreach my $delete (@deletions) {
			my @secondsplit = split/,/,$delete;
			$delete =~ s/,/:/g;
			print TOTAL "$delete;";

			if (($secondsplit[1]<= 20) |($secondsplit[2]<=20)){
				print FIRST "$delete;";
			} elsif ((($secondsplit[1]|$secondsplit[2]) > 20) or (($secondsplit[1]|$secondsplit[2]) <= 40)) {
				print SECOND "$delete;";
			} elsif (($secondsplit[1] >= 40) && ($secondsplit[1] <= 60)) {		#this should only print here if the indel starts in the final third, but it can end afterward. 
				print THIRD "$delete;";
			} 
		}
		foreach my $maturedelete (@maturedeletions) {
			$maturedelete =~ s/,/:/g;
			print MATURE "$maturedelete;";
		}
	}
	print TOTAL ",";
	print FIRST ",";
	print SECOND ",";
	print THIRD ",";	
	print MATURE ",";
	foreach my $snp (@snps){
		my @snpsplit = split/,/,$snp;
		my $snpjoin = join(':',@snpsplit);
		#order: CTP position, new residue, old residue, substitution 
		
		my $position = $snpsplit[0];
		if ($position <= 20) {
			print FIRST "$snpjoin;";
			print TOTAL "$snpjoin;";
		} elsif (($position > 20) && ($position <= 40)) {
			print SECOND "$snpjoin;";
			print TOTAL "$snpjoin;";
		} elsif (($position > 40) && ($position <= 60)) {
			print THIRD "$snpjoin;";
			print TOTAL "$snpjoin;";
		} 
	}
	foreach my $mature (@maturesnps) {
		my @snpsplit = split/,/,$mature;
		my $snpjoin = join(':',@snpsplit);
		print MATURE "$snpjoin;";
		print MATURE "$snpjoin;";
	}
	print FIRST ",";
	print SECOND ",";
	print THIRD ",";
	print TOTAL ",";
	print MATURE ",";
	foreach my $test (@totalindels) {
		print TOTAL "$test;";
	}
	
	#FINAL prints: nucleotide composition
	foreach my $acid (@aminoacids) {
		print FINAL $snpinsert{$acid};
		print FINAL ",";
	}
	print FINAL ",";
	foreach my $acid (@aminoacids) {
		print FINAL $snpdelete{$acid};
		print FINAL ",";
	}
	print FINAL ",";
	foreach my $acid (@aminoacids) {
		print FINAL $indelinsert{$acid};
		print FINAL ",";
	}
	print FINAL ",";
	foreach my $acid (@aminoacids) {
		print FINAL $indeldelete{$acid};
		print FINAL ",";
	}
	print FINAL ",";
	foreach my $acid (@aminoacids) {
		print FINAL $totalinsert{$acid};
		print FINAL ",";
	}
	print FINAL ",";
	foreach my $acid (@aminoacids) {
		print FINAL $totaldelete{$acid};
		print FINAL ",";
	}
	print FINAL "\n";
	print FIRST "\n";
	print SECOND "\n";
	print THIRD "\n";
	print TOTAL "\n";
	print MATURE "\n";
	
	
}






sub SNP {
	my ($ctp,$nonctp) = @_;
	#print "my ctp is $ctp and my non is $nonctp\n";
	my $ctpident ='';
	my $nonident ='';
	if ($ctp =~ m/(G|A|V|L|M|I)/i) {
		$ctpident = "N"; 	#nonpolar
	} elsif ($ctp =~ m/(F|Y|W)/i) {
		$ctpident = "R";	#aromatic
	} elsif ($ctp =~ m/(D|E)/i) {
		$ctpident = "A"; 	#acidic
	} elsif ($ctp =~ m/(K|R|H)/i) {
		$ctpident = "B"; 	#basic
	} elsif ($ctp =~ m/(S|T|C|P|N|Q)/i) {
		$ctpident = "P"; 	#polar 
	} else {
		print "Unidentified ctp character $ctp detected in subroutine; non is $nonctp\n";
	}
	
	if ($nonctp =~ m/(G|A|V|L|M|I)/i) {
		$nonident = "N"; 	#nonpolar
	} elsif ($nonctp =~ m/(F|Y|W)/i) {
		$nonident = "R";	#aromatic
	} elsif ($nonctp =~ m/(D|E)/i) {
		$nonident = "A"; 	#acidic
	} elsif ($nonctp =~ m/(K|R|H)/i) {
		$nonident = "B"; 	#basic
	} elsif ($nonctp =~ m/(S|T|C|P|N|Q)/i) {
		$nonident = "P"; 	#polar 
	} else {
		print "Unidentified non character $nonctp detected in subroutine; ctp is $ctp\n";
	}
	
	
	my $result;
	if ($ctpident eq $nonident) {
		$result = "C";
	} else {
		$result = "N";
	}
	$result = $result . ",$ctpident,$nonident";	#each SNP will contain information about whether it is conserved or not, and what the original and new residue class are.
	return $result;
}