#!/usr/bin/perl
#plastidclusterfinder2.0.pl
use warnings;
use strict;

#the purpose of this program is to find all clusters containing chloroplast-targeted sequences
#the program will also perform the STATS subroutine
#counts for unique clusters of each species will be collated.
#counts for ALL, ALL-1, ALL-2, and ALL-3 will also be performed. 


#implement a test for clusters which are shared with Arabidopsis. 

####
	#version 2.0 is intended to implement new parameters for calling something a "shared" cluster
		#at least 1 monocot + 1 dicot are "plastid-targeted"
		#at least 13/15 species
	#also adds in a new output file for shared clusters
####

print "Please input merged CSV to input names for: \n";
my $csv = <STDIN>;
$csv =~ s/"//g;
chomp $csv;

open (CSV, "<", $csv) or die "couldn't open file $csv for reading: $!\n";
my $write = $csv;
$write =~ s/.csv/_PlastidialClusters.csv/i;
open (WRITE, ">", $write) or die "couldn't open file $write for writing: $!\n";

print "Please input species header abbreviations separated by spaces: \n";
my $speciesinput = <STDIN>;
chomp $speciesinput;
my @species = split/ /, $speciesinput;
my $scalarspecies = scalar @species;


my $results = $csv;
$results =~ s/.csv/_PLASTIDIAL-RESULTS.csv/i;
my $totalresults = $csv;
$totalresults =~ s/.csv/_TOTAL-RESULTS.csv/i;
open (TOTAL, ">", $totalresults) or die "couldn't open file $totalresults for writing: $!\n";


my $i = 0;

my @count;
my @arabidopsis;
my @speciesclusters;
my @speciesplastidialclusters;
my @plastidialarabidopsis;
my @singletons;


my $x = 0;
foreach my $species (@species) {
	$count[$x]=0;
	$singletons[$x]=0;
	$arabidopsis[$x]=0;
	$speciesplastidialclusters[$x]=0;
	$plastidialarabidopsis[$x]=0;
	$speciesclusters[$x]=0;
	$x++;
}
$count[$x] = 0;	#ALL species
$count[$x+1] = 0;	#-1 species
$count[$x+2] = 0;	#-2 species
$count[$x+3] = 0;	#-3 species
$count[$x+4] = 0;	#-4 species
$count[$x+5] = 0;	#-5 species
$count[$x+6] = 0;	#ALL monocots
$count[$x+7] = 0;	#monocots-1
$count[$x+8] = 0;	#monocots-2
$count[$x+9] = 0;	#monocots-3
$count[$x+10] = 0;	#ALL dicots
$count[$x+11] = 0;	#dicots-1
$count[$x+12] = 0;	#dicots-2
$count[$x+13] = 0;	#dicots-3
$count[$x+13] = 0;	#Majority of species have plastidial representation
$count[$x+14] = 0; #>60% (9+) of species have plastidial representation



my @monocots = qw(Antam Bradi Orysa Panvi Setit Sorbi);
my $monocotcheck = join("|",@monocots);
my @dicots = qw(Artha Frave Glyma Maldo Poptr Prupe Solly Vitvi);
my $dicotcheck = join("|",@dicots);

#Ambtr Antam Artha Bradi Frave Glyma Maldo Orysa Panvi Poptr Prupe Setit Solly Sorbi Vitvi


my @header;
push @header, "Cluster Number";
push @header, "Total Sequences";
push @header, "Plastidial Sequences";
foreach my $specie (@species) {
	my $columnheader = "$specie Unique";
	push @header, $columnheader;
}
push @header, "Unique Cluster";
push @header, "15 Species";
push @header, "14+ Species";
push @header, "13+ Species";
push @header, "12+ Species";
push @header, "11+ Species";
push @header, "10+ Species";
push @header, "ALL Monocots";
push @header, "Monocots-1";
push @header, "Monocots-2";
push @header, "Monocots-3";
push @header, "ALL Dicots";
push @header, "Dicots-1";
push @header, "Dicots-2";
push @header, "Dicots-3";
push @header, "Majority of Species";
push @header, "60% of Species";

	#the first n=scalar(@species) values will record the number of times a unique plastidial cluster from species[$x] has been detected
	#the next six will record the number of clusters containing 15, >13, >12, >11,  >10, and >9 sequences

	
my $stats = $csv;
$stats =~ s/.csv/_STATS.csv/i;
open (NUMBERS, ">", $stats) or die "couldn't open file $stats for writing: $!\n";
my $unique = $csv;
$unique =~ s/.csv/_SpeciesUniquePlastidClusters.csv/i;
open (UNIQUE, ">", $unique) or die "couldn't open file $unique for writing: $!\n";
my $shared = $csv;
$shared =~ s/.csv/_SharedPlastidialClusters.csv/i;
open (SHARED, ">", $shared) or die "couldn't open file $shared for writing: $!\n";
print SHARED "Cluster ID,Cluster Population, Plastidial Population, Plastidal-Dicots, Plastidial-Monocots,";

my $amborella = $csv;
$amborella =~ s/.csv/_Shared-Amborella-Plastidial.csv/i;
open (AMBOR, ">", $amborella) or die "couldn't open file $amborella for writing: $!\n";
my $amborellaall = $csv;
$amborellaall =~ s/.csv/_Shared-Amborella-ALL.csv/i;
open (AMBORALL, ">", $amborellaall) or die "couldn't open file $amborellaall for writing: $!\n";


print AMBOR "Cluster ID,Cluster Population, Plastidial Population, Plastidal-Dicots, Plastidial-Monocots,";
print AMBORALL "Cluster ID,Cluster Population, Plastidial Population, Plastidal-Dicots, Plastidial-Monocots,";
	local $, = ",";
	for (my $r=0; $r<=500; $r++){
		print SHARED "seq$r,";
		print AMBOR "seq$r,";
		print AMBORALL "seq$r,";
	}

				

my $clustercount = 0;
my $largestcluster = 0;
my $largestclusterid = '';
my $plastidclustercount = 0;
my $largestplastidcluster = 0;
my $largestplastidclusterid = '';
my $largestplastidtotal = 0;

local $, = ",";
print WRITE @header;
print TOTAL @header;
print UNIQUE @header;

while (<CSV>) {
	my @binary;
	$binary[0] = 0; #UNIQUE
	$binary[1] = 0;	#ALL species
	$binary[2] = 0;#-1 species
	$binary[3] = 0;#-2 species
	$binary[4] = 0;#-3 species
	$binary[5] = 0;#-4 species
	$binary[6] = 0;#-5 species
	$binary[7] = 0;#ALL monocots
	$binary[8] = 0;#monocots-1
	$binary[9] = 0;#monocots-2
	$binary[10] = 0;#monocots-3
	$binary[11] = 0;#ALL dicots
	$binary[12] = 0;#dicots-1
	$binary[13] = 0;#dicots-2
	$binary[14] = 0;#dicots-3
	
	my $line = $_;
	chomp $line;
	my @headers = split/,/,$line;
	my $clusterid = shift @headers;
	if (scalar @headers > $largestcluster) {
		$largestclusterid = $clusterid;
		$largestcluster = scalar @headers;
	}
	$clustercount++;
	my $clusterpop = scalar @headers;
	
	my $j = 0;
	#do species check
	#check for clustering with Arabidopsis
	foreach my $species2 (@species) {
		if (grep(m/$species2/i,@headers)) {
			$speciesclusters[$j]++;
			if (grep(m/Artha/i,@headers)) {
				$arabidopsis[$j]++;
			}
		}
		$j++;
	}
	
	if (grep(/c,|c$/,$line)) {
		$plastidclustercount++;
		my @plastidial = grep/c$/,@headers;	
		if (scalar @plastidial > $largestplastidcluster) {
			$largestplastidclusterid = $clusterid;
			$largestplastidcluster = scalar @plastidial;
			$largestplastidtotal = scalar @headers;
		}
		my @split = split/,/,$line;
		my @statistics = STATS(\@plastidial,\@species);
		my @totalstatistics = STATS(\@headers,\@species);
			#perform tests for unique clusters and >9+ clusters
		my @substats = @statistics;
		my $fullseqs = shift @substats;	#save the # of headers metric; will be useful for identifying good candidates.
		my $sum = 0;
		
		#do species plastidial check
		#check for plastidial clustering with Arabidopsis
		
		my $g = 0;
		foreach my $species3 (@species) {
			if (grep(m/$species3/i,@plastidial)) {
				$speciesplastidialclusters[$g]++;
				if (grep(m/Artha/i,@plastidial)) {
					$plastidialarabidopsis[$g]++;
				}
			}
			$g++;
		}
		
		
		
		
		foreach my $substat (@substats) {
			if ($substat > 0) {
				$substat = 1;
			}
			$sum = $sum + $substat;
		}
		my $uniquebinary = 0;
			if ($sum == 1) {
				$uniquebinary = 1;
				$binary[0] = 1;		#indicates the cluster is a unique plastidial one
				my $n = 0;
				#check which species is the one unique one
				foreach my $stat (@substats) {
					if ($stat == 1) {
						$count[$n]++;
					}
					$n++;
				}
				if (scalar @headers == scalar @plastidial) {
					#find species containing the singleton
					$n = 0;
					foreach my $spec (@species) {
						if (grep/$spec/, @plastidial) {
							last;
						}
						$n++;
					}
					$singletons[$n]++;
				}
			}
		if ($sum == $scalarspecies) {
			$count[$scalarspecies]++;
			$binary[1] = 1;
		}
		if ($sum >= ($scalarspecies-1)) {
			$count[$scalarspecies+1]++;
			$binary[2] = 1;
		}
		if ($sum >= ($scalarspecies-2)) {
			$count[$scalarspecies+2]++;
			$binary[3] = 1;
		}
		if ($sum >= ($scalarspecies-3)) {
			$count[$scalarspecies+3]++;
			$binary[4] = 1;
		}
		if ($sum >= ($scalarspecies-4)) {
			$count[$scalarspecies+4]++;
			$binary[5] = 1;
		}
		if ($sum >= ($scalarspecies-5)) {
			$count[$scalarspecies+5]++;
			$binary[6] = 1;
		}
		
		#monocot count
		my $monocotcount = 0;
		
		unless (grep/$dicotcheck/,@plastidial) {
			foreach my $monocot (@monocots) {
				if (grep/$monocot/i,@plastidial) {
					$monocotcount++;
				}
			}
		}
		if ($monocotcount == 6) {
			$count[$x+6]++;
			$binary[7] = 1;
		}
		if ($monocotcount >= 5) {
			$count[$x+7]++;
			$binary[8] = 1;
		}
		if ($monocotcount >= 4) {
			$count[$x+8]++;
			$binary[9] = 1;
		}
		if ($monocotcount >= 3) {
			$count[$x+9]++;
			$binary[10] = 1;
		}
		
		#dicot count
		my $dicotcount = 0;
		unless (grep/$monocotcheck/,@plastidial) {
			foreach my $dicot (@dicots) {
				if (grep/$dicot/i,@plastidial) {
				$dicotcount++;
				}
			}
		}
				
		if ($dicotcount == 8) {
			$count[$x+10]++;
			$binary[11] = 1;
		}
		if ($dicotcount >= 7) {
			$count[$x+11]++;
			$binary[12] = 1;
		}
		if ($dicotcount >= 6) {
			$count[$x+12]++;
			$binary[13] = 1;
		}
		if ($dicotcount >= 5) {
			$count[$x+13]++;
			$binary[14] = 1;
		}

		if ($sum >= 8) {
			$count[$x+14]++;
		}
		if ($sum >= 9) {
			$count[$x+15]++;
		}
		$dicotcount = 0;
		$monocotcount = 0;
		
		my $totalsum = 0;
		foreach my $totalstat (@totalstatistics) {
			if ($totalstat > 0) {
				$totalstat = 1;
			}
			$totalsum = $totalsum + $totalstat;
		}
		
		foreach my $monocot (@monocots) {
			if (grep/$monocot/i,@plastidial) {
				$monocotcount++;
			}
		}
		foreach my $dicot (@dicots) {
			if (grep/$dicot/i,@plastidial) {
			$dicotcount++;
			}
		}
		my $plastidialslice = scalar @plastidial;
		
		if (($dicotcount > 0) && ($monocotcount > 0)){
			if ($totalsum >= 13) {
				print SHARED $clusterid . ",";
				print SHARED $clusterpop . ",";
				print SHARED $plastidialslice . ",";
				print SHARED $dicotcount . ",";
				print SHARED $monocotcount . ",";
				local $, = ",";
				print SHARED @headers;
				print SHARED "\n";
				if (grep(/ambtr/i, @plastidial)) {
					print AMBOR $clusterid . ",";
					print AMBOR $clusterpop . ",";
					print AMBOR $plastidialslice . ",";
					print AMBOR $dicotcount . ",";
					print AMBOR $monocotcount . ",";
					local $, = ",";
					print AMBOR @headers;
					print AMBOR "\n";
				}
				if (grep(/ambtr/i, @headers)) {
					print AMBORALL $clusterid . ",";
					print AMBORALL $clusterpop . ",";
					print AMBORALL $plastidialslice . ",";
					print AMBORALL $dicotcount . ",";
					print AMBORALL $monocotcount . ",";
					local $, = ",";
					print AMBORALL @headers;
					print AMBORALL "\n";
				}
			}
		}	
		push @statistics, @binary;
		local $, = ",";

		print WRITE "\n";
		print WRITE $clusterid . ",";
		print WRITE $clusterpop . ",";
		print WRITE @statistics;
		print WRITE ",";
		print WRITE @plastidial;

		print TOTAL "\n";
		print TOTAL $clusterid . ",";
		print TOTAL $clusterpop . ",";
		print TOTAL @statistics;
		print TOTAL ",";
		print TOTAL @headers;
		if ($uniquebinary == 1) {

			print UNIQUE "\n";
			print UNIQUE $clusterid . ",";
			print UNIQUE $clusterpop . ",";
			print UNIQUE @statistics;
			print UNIQUE ",";
			print UNIQUE @headers;
			}
	}
}



close WRITE;

my $y = 0;
foreach my $spec (@species) {
	print NUMBERS $spec . " has ";
	print NUMBERS $count[$y];
	print NUMBERS " unique headers of which ";
	print NUMBERS $singletons[$y];
	print NUMBERS " are singletons (have no other cluster sequences)\n";
	print NUMBERS "$spec has " . $speciesclusters[$y] . " clusters of which " . $arabidopsis[$y] . " also cluster with Arabidopsis\n";
	print NUMBERS "$spec has " . $speciesplastidialclusters[$y] . " plastidial clusters of which " . $plastidialarabidopsis[$y] . " also cluster with Arabidopsis\n\n";
	
	
	$y++;
}
print NUMBERS $count[$y] . " clusters have plastidial sequences for all species\n";
$y++;
my $current = $scalarspecies - 1;
print NUMBERS $count[$y] . " clusters have plastidial sequences for " . $current . " species\n";
$y++;
$current = $current - 1;
print NUMBERS $count[$y] . " clusters have plastidial sequences for " . $current . " species\n";
$y++;
$current = $current - 1;
print NUMBERS $count[$y] . " clusters have plastidial sequences for " . $current . " species\n";
$y++;
$current = $current - 1;
print NUMBERS $count[$y] . " clusters have plastidial sequences for " . $current . " species\n";
$y++;
$current = $current - 1;
print NUMBERS $count[$y] . " clusters have plastidial sequences for " . $current . " species\n";
$y++;
print NUMBERS $count[$y] . " monocot-specific clusters contain ALL monocots\n";
$y++;
print NUMBERS $count[$y] . " monocot-specific clusters contain 5+ monocots\n";
$y++;
print NUMBERS $count[$y] . " monocot-specific clusters contain 4+ monocots\n";
$y++;
print NUMBERS $count[$y] . " monocot-specific clusters contain 3+ monocots\n";
$y++;
print NUMBERS $count[$y] . " dicot-specific clusters contain ALL dicots\n";
$y++;
print NUMBERS $count[$y] . " dicot-specific clusters contain 7+ dicots\n";
$y++;
print NUMBERS $count[$y] . " dicot-specific clusters contain 6+ dicots\n";
$y++;
print NUMBERS $count[$y] . " dicot-specific clusters contain 5+ dicots\n";
$y++;
print NUMBERS $count[$y] . " clusters contain a majority of species with plastidial sequences\n";
$y++;
print NUMBERS $count[$y] . " clusters contain greater than 60% of species\n";

print NUMBERS "The largest cluster is $largestclusterid with $largestcluster sequences\n";
print NUMBERS "The largest plastidial cluster is $largestplastidclusterid with $largestplastidcluster plastidial sequences and $largestplastidtotal total sequences\n";
	
sub STATS {
	#sub counts the number of headers, and counts the number of matches for each species. 
	my ($headers_ref, $species_ref) = @_;
	my @headers = @{ $headers_ref };
	my @species = @{ $species_ref };
		#dereference arrays
	
	#change local delimiter to a comma (CSV format)
	my @numbers;
	push @numbers, (scalar @headers);

	
	#does a grep for each species' header ID, and pushes the counted number to "@numbers"
	foreach my $specie (@species) {
		my @grep = grep (m/$specie/g, @headers);
		
		#tests for array elements in @grep and returns number of matches OR "0" if undefined
		if (@grep) {
			push @numbers, (scalar @grep);
		} else {
			push @numbers, "0";
		}
}		
local $, = ",";

return @numbers;
}