#!/usr/bin/perl
#seqrand.plx
use strict;
use warnings;

#program is designed to take formatted UCLUST output files and select random sequences for each file
	#after selecting, will write to master list
	#once all files are read, master list will be re-sorted based on length, then the complete FASTA will be appended to it
	
#purpose of program is to create random seed files for the UCLUST platform to decrease bias towards set centroids



print "\t\t\tRunning perl version $^V\n" ;
print "\t\t\tProgram Developed by Ryan Christian\n";
print "\t\t\tOriginal coding: 7-22-2016\n";
print "\t\t\tLast Modified: 2-15-2017\n";



#parse through algorithm x number of times
my $counter = 0;
print "Please indicate number of iterations: \n";
my $iterate = <STDIN>;
chomp $iterate;


print "Please indicate master FASTA file: \n";
my $masterfasta = <STDIN>;
chomp $masterfasta;
$masterfasta =~ s/"//g;



print "Please indicate master directory: \n";
my $dir = <STDIN>;
chomp $dir;
$dir =~ s/"//g;
chdir $dir;



#open master directory and get filenames

if (-d $dir) {
	print "Reading from directory $dir.\n";
}	else {
	print "$dir is not a directory! \n";
	die;
}

opendir(DH, $dir);
my @dirfiles = readdir(DH);
closedir(DH);

my @masterfiles = grep(m/cluster/i, @dirfiles);

my $ctporprot;
if ($masterfasta =~ m/CTP/i) {
	$ctporprot = "CTP";
	}	else {
	$ctporprot = "PROT";
}

#main program: runs the GETSEED and WRITE subroutines between 1 and x times
while ($counter < $iterate) {
	$counter++;
	
	my %randfiles;
	my $seedtemp;
	my @sortedrand;

	%randfiles = GETSEED(@masterfiles);
	print "\nSequence Randomization $counter complete: \t" . localtime . "\n";
	
	$seedtemp = WRITE($counter, %randfiles);
	print "Random seeds written to $seedtemp: \t" . localtime . "\n";
	
	@sortedrand = SORT(%randfiles);
	print "Length sorting of random seeds $counter complete: \t" . localtime . "\n";
	
	APPEND($masterfasta, $ctporprot, $counter, @sortedrand);
	print "Seed file written to SEED$counter.fasta: \t" . localtime . "\n";
	print "Iteration $counter is complete: \t" . localtime . "\n";
}

#SUBS!!!

#subroutine to get directory and grab filenames


sub GETSEED {
	my @files = @_;
	my %seqhash;

	foreach my $cluster (@files) {
		open (CLUSTER, $cluster) or die;
		my @lines = <CLUSTER>;
		close CLUSTER;
		
		my @headers = grep(/(^>\w{9}\d*)/, @lines);

		my @seqs = grep(/^\w/, @lines);				#need to fix; some seqs only 5 amino acids long
		my $number = scalar @headers; 
		chomp @headers;
		chomp @seqs;
		
	
			my $seed = int(rand($number));
			$seqhash{$headers[$seed]} = $seqs[$seed];	
	}
	return %seqhash;
}

#seedseqs(x).txt is a file containing the unordered randomly selected seeds
sub WRITE {
	my ($counter, %randseeds) = @_;
	my $seedtemp = "seedseqs$counter.txt";
	open (WRITING1, '>', $seedtemp);
		{
		local $, = "\n";
		print WRITING1 %randseeds;
		}
	close WRITING1;	
	return $seedtemp;
	}

#SORT subroutine joins header/seq and creates an array that is length-sorted. Splits back into header/fasta and returns split array.
sub SORT {

#pass hash of seedbatch master file into subroutine
	my %unsortedseqs = @_;
	my @sortedrand;
	foreach my $key (sort {length ($unsortedseqs{$b}) <=> length ($unsortedseqs{$a})} keys %unsortedseqs) {
		push @sortedrand, "$key", "$unsortedseqs{$key}";
	}
	return @sortedrand;	
}

#APPEND subroutine takes sorted array of seed seqs, then adds the master seqs on to this list
sub APPEND {

my ($masterfasta, $ctporprot, $counter, @sorted) = @_;
open (MASTER, $masterfasta);

my @master = <MASTER>;
close MASTER;
my %masterseqs;

my @mheaders = grep(/^>/, @master);
my @mseqs = grep(/^\w/, @master);
chomp @mheaders;
chomp @mseqs;
@masterseqs{@mheaders} = @mseqs;


my @sortedmaster = SORT (%masterseqs);
chomp @sortedmaster;

push(@sorted, @sortedmaster);

my $seedfile = "$ctporprot-seed$counter.fasta";
open (FINAL, ">", $seedfile);
	{
	local $, = "\n";
	print FINAL @sorted;
	}
close FINAL;
}



	
	#Program guts:
		#open dir
		#pick randoms
			#write each to temp file
		#sort randoms
		#append master fasta sequences back on
			#sort masta fasta first
			#then append
	
	

