#!/usr/bin/perl
#uclustsystemcall.plx
use strict;
use warnings;

#This program is designed to run UCLUST/USEARCH with a user-specified identity threshold on a directory of fasta input files. 
		#program will run UCLUST, then immediately condense results down into .csv format and delete the generated cluster files	
		#program will ignore the centroid file. 
		
#Please note that UCLUST exe file must be in the same directory as the FASTA inputs. 



print "Please specify home directory of UCLUST program: \n";
my $homedir = <STDIN>;
chomp $homedir;
$homedir =~ s/"//g;


print "Please specify cluster directory\n";
my $clusterhome = <STDIN>;
chomp $clusterhome;
$clusterhome =~ s/"//g;

print "Please specify identity threshold (Between 0.0 and 1.0)\n";
my $id = <STDIN>;
chomp $id;

chdir $clusterhome;
opendir (DIR, $clusterhome);
my @files = readdir(DIR);
my @fastas = grep(m/(seed\d*.fasta)$/, @files);

chdir $homedir; 

my $datatype = "PROT";

	
foreach my $fasta (@fastas) {

	(my $i) = ($fasta =~ m/(\d+)/gi);
		#retrieve run number for proper labeling of named files

	system("usearch9.2.64_win32", "-cluster_fast", "$fasta", "-sort", "other", "-id", "$id", "-centroids", "Centroids_Run$i.fasta", "-clusters", "Clusters_Run$i");
		#-sort other is the default argument, but is specified just in case
		#this command uses inputs in the order they appear, not based on length or size. 
	CONDENSE($fasta, $homedir, $i);
}


	#================================================================================================================
											#UCLUST CONDENSER SUBROUTINE
	#================================================================================================================
	
	#note: UCLUST condenser does NOT copy sequences. Sequences are only recorded in Run0.
	
sub CONDENSE {	
	my ($fasta, $homedir, $i) = @_;
	$fasta =~ s/\.fasta//i;
	opendir(DH, $homedir);
	my @files = readdir(DH);
	my @clusters = grep(m/^Clusters/, @files);
	closedir(DH);

	#declare the species (will be used in STATS sub)
	my @species = qw(Antam Artha Bradi Frave Maldo Orysa Prupe Poptr Panvi Sorbi Setit Solly Vitvi);

	my $out = "$fasta.csv";
	open (OUT, ">", $out) or die "Couldn't open $out for writing: $!";

	my @tableheader = ("Cluster Name", "Cluster Population", "Antam", "Artha", "Bradi", "Frave", "Maldo", "Orysa", "Prupe", "Poptr", "Panvi", "Sorbi", "Setit", "Solly", "Vitvi");

	for (my $i = 1; $i <= 500; $i++) {
		push @tableheader, "Seq$i";
	}

	local $, = ",";
	#print @tableheader;
	print OUT @tableheader;
	print OUT "\n";

	foreach my $cluster (@clusters) {

		if($cluster =~ /^clusters/i) {
				#I can't grep for /$.fasta/ because the files are extensionless after UCLUST runs. 
		#	print $file;
			open (CLUSTER, $cluster) or die "Can't open $cluster for read/write: $!\n";
			my @lines = <CLUSTER>;
			close CLUSTER;
		#	print @lines;
			
			my @headers = grep (m/>\w{9}\d*/, @lines);
			my @numbers = STATS (\@headers, \@species);

			chomp (@headers);
			chomp (@numbers);
			local $, = ',';
			
			print OUT "$cluster" . ",";
			print OUT @numbers;
			print OUT ",";
			print OUT @headers;
			print OUT "\n";
			#delete each cluster file as script proceeds
			unlink $cluster;
		} else {
			next;
		}
	}
	close OUT;
	my $centroids = "Centroids_Run$i.fasta";
	EXTRACT($centroids);
}	
	
#================================================================================================================
											#SUBROUTINES
#================================================================================================================

sub STATS {
	#sub counts the number of headers, and counts the number of matches for each species. 
	my ($headers_ref, $species_ref) = @_;
	my @headers = @{ $headers_ref };
	my @species = @{ $species_ref };
		#these are dereferencers, in case I forget in the future!
	
	#change local delimiter to a comma (CSV format)
	my @numbers;
	push @numbers, (scalar @headers);

	
	#does a grep for each species' header ID, and pushes the counted number to "@numbers"
	foreach my $specie (@species) {
		my @grep = grep (m/>$specie/g, @headers);
		
		#tests for array elements in @grep and returns number of matches OR "0" if undefined
		if (@grep) {
			push @numbers, (scalar @grep);
		} else {
			push @numbers, "0";
		}
}		
return @numbers;
}


sub EXTRACT {

	my $centroidfile = shift;

	open (CENTROID, "<", $centroidfile) or die "couldn't open file $centroidfile for headerextraction: $!\n";
	my @centroids = <CENTROID>;
	close CENTROID;
	local $, = "\n";
	my @headers = grep(m/^>/, @centroids);
	chomp @headers;
	#print @headers;
	open (CENTROID, ">", $centroidfile) or die "couldn't open file $centroidfile for overwrite: $!\n";
	print CENTROID @headers;
	}
