#!usr/bin/perl
#uniqueconsensus6.pl
use warnings;
use strict;
use List::UtilsBy qw(extract_by);

#this perl script is designed to detect if each potential "unique plastid targeted" candidate is found in BLAST only, UCLUST only, or both. 
	#start by inputting both "unique clusters" files as separate arrays
	#create a merged array by using "UNIQUE" subroutine and sort lexically
	#iterate through merged list and detect hits in both datasets
	
#one issue that needs to be addressed: non-singleton clusters (single species)



#something to code in:
	#parse down BLAST and UCLUST files to the plastidial sequences only
	#check for overlap with multiple headers
	
	
#in version 4, I added explanations for different loops and steps
	#also realized that the BLAST and UCLUST sections at the bottom where clusters were expanded was incorrect; they were backwards. This likely explains the complete mess that clusters ended up in...
	#explains why so many of the "Blast" and "uclust" only seqs were not single-species cTP; they were grabbing from the wrong dataset oops. 
	
	
print "Please indicate BLAST cluster file parsed for species-unique plastidial sequences:\n";
my $blast = <STDIN>;
chomp $blast;
$blast =~ s/"|'//g;


print "Please indicate UCLUST file parsed for species-unique plastidial sequences:\n";
my $uclust = <STDIN>;
chomp $uclust;
$uclust =~ s/"|'//g;



print "Detecting Consensus Clusters...\n";

open (BLAST, "<", $blast) or die "Couldn't open $blast for reading: $!";
my @blasts = <BLAST>;
chomp @blasts;
shift @blasts;
close BLAST;



my @parsedblast = @blasts;

my $n = 0;
my %blastsaved;
foreach my $blastline (@parsedblast) {
	chomp $blastline;
	my @bsplit = split/,/,$blastline;
	my $blastclusterid = $bsplit[0];

	my @plastidialblast = grep(/c$/, @bsplit);
	@plastidialblast = sort @plastidialblast;
	@bsplit = sort @bsplit;
	my $newline = join(',',@plastidialblast);
	my $fulline = join(',',@bsplit);
	$newline = $blastclusterid  . "," .  $newline;
	$parsedblast[$n] = $newline;
	$blastsaved{$blastclusterid} = $fulline;
	$n++;
}

open (UCLUST, "<", $uclust) or die "Couldn't open $uclust for reading: $!";
my @uclusts = <UCLUST>;
chomp @uclusts;
shift @uclusts;
close UCLUST;
my @parseduclust = @uclusts;


$n = 0;
my %uclustsaved;
foreach my $uclustline (@parseduclust) {
	chomp $uclustline;
	my @usplit = split/,/,$uclustline;
	my $uclustclusterid = $usplit[0];
	unless ($uclustclusterid) {
		next;
	}
	
	my @plastidialuclust = grep(/c$/, @usplit);
	@plastidialuclust = sort @plastidialuclust;
	@usplit = sort @usplit;
	my $newline = join(',',@plastidialuclust);
	my $fulline = join(',',@usplit);
	$newline = $uclustclusterid . "," . $newline;
	$parseduclust[$n] = $newline;
	$uclustsaved{$uclustclusterid} = $fulline;
	$n++;
}



	
my @merged = @parsedblast;
push @merged, @parseduclust;
 
	#instead of going through each list separately, find every unique cluster. 

#this loop strips headers and at the end finds unique cluster contents
my @semimerged;
my @headerlist;

foreach my $initial (@merged) {			#this loop finds all unique cTP headers in the analysis.
	my @split = split/,/,$initial;
	shift @split;
	foreach my $split (@split) {
		push @headerlist, $split;
	}
	@split = sort @split;
	my $joined = join(',',@split);
	push @semimerged, $joined;
}
@headerlist = UNIQUE(@headerlist);


foreach my $singleheader (@headerlist) {								#this loop collapses all multi-membered clusters into larger clusters
	my @matches = extract_by{m/($singleheader)(,|$)/} @semimerged;			#condenses any clusters that differed between UCLUST and BLAST in terms of plastidial contents. 
	my @clustersplit;
	foreach my $semimatch (@matches) {
		my @semisplit = split/,/,$semimatch;
		push @clustersplit, @semisplit;
	}
	@clustersplit = UNIQUE(@clustersplit);
	my $finaljoin = join(',',@clustersplit);
	push @semimerged, $finaljoin;
	
}


my %finalmerge;
my @uniquelist;

my $unique = 1;
print  "\n";
print scalar @semimerged;
foreach my $semi (@semimerged) {					#this next loop generates cluster names for the semi-merged clusters and creates both an ordered array and a hash associated with that array. 						
	unless ($semi) {
		next;
	}
	my $uniqueid = "UniqueCluster_$unique";
	if ($unique < 10) {
		print $uniqueid;
		print "\n";
		print $semi;
		print "\n";
	}
	push @uniquelist, $uniqueid;
	$finalmerge{$uniqueid} = $semi;
	$unique++;
}
print  "\n";
print scalar @uniquelist;
	

#this loop iterates through the "unique ID's" for cTP clusters and finds all BLAST and UCLUST headers associated with each and every header that's in it among the "parsed" clusters files. 
	#if the program finds that there's only one species with a plastidial sequence, it posts to the "consensus" file
	#otherwise, the program prints to either UCLUST or BLAST only. 
	#additionally, the program keeps track of which cluster contains the most associated sequences and labels it "best"

my @blastonly;
my @uclustonly;
my @clusterconsensus;

foreach my $key (@uniquelist) {	
	my $line = $finalmerge{$key};
	my @finalsplit = split/,/,$line;
	@finalsplit = sort @finalsplit;
	my $query = join('(,|$)|',@finalsplit);
	my $scalar = scalar @finalsplit;
	if ($scalar == 0) {
		next;
	}

	my $blasthit = 0;
	my $uclusthit = 0;
	#check BLAST, number, find BLAST cluster ID's
	my @blastgrep = grep(m/$query/, @blasts);
	my @blastseqs;
	my @blastheaders;
	my $blastscalar = scalar @blastgrep;
	if ($blastscalar > 0) {							#this keeps track of each individual BLAST cluster found to contain one or more of the queried headers. 
		foreach my $bgrep (@blastgrep) {
			$blasthit++;
			my @localsplit = split/,/,$bgrep;
			my $clusterhead = shift @localsplit;
			push @blastheaders, $clusterhead;
			push @blastseqs,@localsplit;
		}
		
	}
	@blastseqs = UNIQUE(@blastseqs);
	my $blastclustercontents = join(',',@blastseqs);
	my $bclusters = join(',',@blastheaders);
	
	
	#check UCLUST, number, find UCLUST cluster ID's
	my @uclustgrep = grep(m/$query/, @uclusts);
	my @uclustseqs;
	my @uclustheaders;
	my $uclustscalar = scalar @uclustgrep;
	if ($uclustscalar > 0) {					#this keeps track of each individual UCLUST cluster found to contain one or more of the queried headers.
		foreach my $ugrep (@uclustgrep) {
			$uclusthit++;
			my @localsplit = split/,/,$ugrep;
			my $clusterhead = shift @localsplit;
			push @uclustheaders,$clusterhead;
			push @uclustseqs,@localsplit;
		}
	} 
	@uclustseqs = UNIQUE(@uclustseqs);
	my $uclustclustercontents = join(',',@uclustseqs);
	my $uclusters = join(',',@uclustheaders);	
	
	my $fullline;
	
	if (($blastscalar > 0) && ($uclustscalar > 0)) {		#UNIQUE clusters for both
		my @mergedclusters = @blastseqs;
		push @mergedclusters, @uclustseqs;
		@mergedclusters = UNIQUE(@mergedclusters);
		my $totalclusters = join(',',@mergedclusters);
		$fullline = "$key\t$line\t$bclusters\t$blastclustercontents\t$uclusters\t$uclustclustercontents\t$totalclusters";
		push @clusterconsensus,$fullline;
	} elsif (($blastscalar > 0) && ($uclustscalar == 0)) {	#only unique in BLAST
		$fullline = "$key\t$line\t$bclusters\t$blastclustercontents\t-\t-\t$blastclustercontents";
		push @blastonly, $fullline;
	} elsif (($blastscalar == 0) && ($uclustscalar > 0)) {	#only unique in UCLUST
		$fullline = "$key\t$line\t-\t-\t$uclusters\t$uclustclustercontents\t$uclustclustercontents";
		push @uclustonly, $fullline;
	} else {
		print "$key failed to be sorted into a file\n";
	}
}

my $keyline = "Unique Cluster Name\tUnique Plastidial Headers\tBLAST0.90 Cluster ID\tBLAST0.90 Plastidial Headers\tUCLUST0.90 Cluster ID\tUCLUST0.90 Plastidial Headers\tTotal Headers";
#example line: UniqueCluster_1 Solly_seq31971c	V5_BLAST_0.9SELF0.4OUT-REDO4-CLUSTER_142846,	Solly_seq31971c	1	Solly_seq31971c	-	-	0	 Panvi_seq103917, Glyma_seq47482, Solly_seq24980c, Setit_seq7105, Solly_seq31971c, Ambtr_seq16495c, Sorbi_seq30983c, Panvi_seq100870c, Poptr_seq735, Vitvi_seq37175c, Frave_seq8394, Glyma_seq58406, Bradi_Seq30373c, Prupe_Seq31898c, Maldo_seq19757c, Glyma_seq29806c, Vitvi_seq37174c, Vitvi_seq37172c, Panvi_seq113146c, Orysa_seq24625c, Maldo_seq43059c, Poptr_seq58865, Poptr_seq736, Vitvi_seq37173c, Artha_Seq32960c

local $, = "\n";
my $blastonly = "V5_BLASTonly_0.90expand-uniques.tab";
open (BO, ">", $blastonly) or die "Couldn't open $blastonly for writing: $!";
print BO $keyline . "\n";
print BO @blastonly;
close BO;


my $uclustonly = "V5_UCLUSTonly_0.90expand-uniques.tab";
open (UO, ">", $uclustonly) or die "Couldn't open $uclustonly for writing: $!";
print UO $keyline . "\n";
print UO @uclustonly;
close UO;


my $consensus = "V5_CONSENSUS_UCLUST0.90-BLAST0.90-uniques.tab";
open (CO, ">", $consensus) or die "Couldn't open $consensus for writing: $!";
print CO $keyline . "\n";
print CO @clusterconsensus;
close CO;



sub UNIQUE {
	my %seen;
    grep !$seen{$_}++, @_;
	my @unique = keys (%seen);
	my $scalar = scalar @_;
	return @unique;
}